#ifndef COMPOSITE_H_
#define COMPOSITE_H_

#include <cstddef> // NULL
#include <iomanip>
#include <iostream>
#include <fstream>
#include <string>
#include <list>

#if defined(MYSERI)
#include <boost/archive/tmpdir.hpp>

#include <boost/archive/text_iarchive.hpp>
#include <boost/archive/text_oarchive.hpp>

#include <boost/serialization/base_object.hpp>
#include <boost/serialization/utility.hpp>
#include <boost/serialization/list.hpp>
#include <boost/serialization/assume_abstract.hpp>
#include <boost/serialization/export.hpp>
#endif


#include "base.h"


/////////////////////////////////////////////////////////////
// a bus route is a collection of bus stops
//
// illustrates serialization of STL collection templates.
//
// illustrates serialzation of polymorphic pointer (bus stop *);
//
// illustrates storage and recovery of shared pointers is correct
// and efficient.  That is objects pointed to by more than one
// pointer are stored only once.  In such cases only one such
// object is restored and pointers are restored to point to it
//
class composite
{

    typedef base * base_pointer;
    std::list<base_pointer> comps;

#if defined(MYSERI)
    friend class boost::serialization::access;
    template<class Archive>
    void serialize(Archive &ar, const unsigned int version)
    {
        // in this program, these classes are never serialized directly but rather
        // through a pointer to the base class bus_stop. So we need a way to be
        // sure that the archive contains information about these derived classes.

        // serialization of stl collections is already defined
        // in the header
        ar & comps;
    }
#endif

public:
    void print();
    composite();
    ~composite();
    void append(base *_bs);
};

#if defined(MYSERI)
BOOST_CLASS_EXPORT(composite)
#endif
  

#endif
